package com.ruoyi.ss.payBill.service.impl;

import java.time.LocalDateTime;
import java.util.List;

import com.ruoyi.common.core.redis.RedisLock;
import com.ruoyi.common.core.redis.enums.RedisLockKey;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.SnowFlakeUtil;
import com.ruoyi.ss.payBill.domain.enums.PayBillBstType;
import com.ruoyi.ss.payBill.domain.enums.PayBillStatus;
import com.ruoyi.ss.timeBill.domain.TimeBillVO;
import com.ruoyi.ss.timeBill.service.TimeBillService;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillPayType;
import com.ruoyi.ss.user.domain.SmUserVo;
import com.ruoyi.ss.user.service.ISmUserService;
import com.ruoyi.common.pay.wx.service.WxPayService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.ss.payBill.mapper.PayBillMapper;
import com.ruoyi.ss.payBill.domain.PayBill;
import com.ruoyi.ss.payBill.domain.PayBillVO;
import com.ruoyi.ss.payBill.domain.PayBillQuery;
import com.ruoyi.ss.payBill.service.PayBillService;
import org.springframework.transaction.support.TransactionTemplate;
import org.springframework.util.CollectionUtils;

/**
 * 支付订单Service业务层处理
 *
 * @author ruoyi
 * @date 2024-07-24
 */
@Service
@Slf4j
public class PayBillServiceImpl implements PayBillService
{
    @Autowired
    private PayBillMapper payBillMapper;

    @Autowired
    private RedisLock redisLock;

    @Autowired
    private WxPayService wxPayService;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private ISmUserService userService;

    @Autowired
    private TimeBillService timeBillService;

    /**
     * 查询支付订单
     *
     * @param payId 支付订单主键
     * @return 支付订单
     */
    @Override
    public PayBillVO selectPayBillByPayId(Long payId)
    {
        return payBillMapper.selectPayBillByPayId(payId);
    }

    /**
     * 查询支付订单列表
     *
     * @param payBill 支付订单
     * @return 支付订单
     */
    @Override
    public List<PayBillVO> selectPayBillList(PayBillQuery payBill)
    {
        return payBillMapper.selectPayBillList(payBill);
    }

    /**
     * 新增支付订单
     *
     * @param payBill 支付订单
     * @return 结果
     */
    @Override
    public int insertPayBill(PayBill payBill)
    {
        payBill.setPayNo(String.valueOf(SnowFlakeUtil.newId()));
        payBill.setCreateTime(DateUtils.getNowDate());
        return payBillMapper.insertPayBill(payBill);
    }

    /**
     * 修改支付订单
     *
     * @param payBill 支付订单
     * @return 结果
     */
    @Override
    public int updatePayBill(PayBill payBill)
    {
        return payBillMapper.updatePayBill(payBill);
    }

    /**
     * 批量删除支付订单
     *
     * @param payIds 需要删除的支付订单主键
     * @return 结果
     */
    @Override
    public int deletePayBillByPayIds(Long[] payIds)
    {
        return payBillMapper.deletePayBillByPayIds(payIds);
    }

    /**
     * 删除支付订单信息
     *
     * @param payId 支付订单主键
     * @return 结果
     */
    @Override
    public int deletePayBillByPayId(Long payId)
    {
        return payBillMapper.deletePayBillByPayId(payId);
    }

    @Override
    public PayBillVO createBill(TimeBillVO bill, Long channelId) {
        // 锁
        String lockKey = bill.getBillNo();
        if (!redisLock.lock(RedisLockKey.PAY_BILL, lockKey)) {
            return null;
        };
        try {
            // 关闭旧的支付订单
            PayBillBstType bstType = PayBillBstType.TIME_BILL;
            boolean close = this.closeByBstId(bill.getBillId(), bstType);
            if (!close) {
                return null;
            }

            // 查询用户
            SmUserVo user = userService.selectSmUserByUserId(bill.getUserId());
            if (user == null) {
                return null;
            }

            // 若关闭成功，创建支付订单
            PayBillVO data = new PayBillVO();
            data.setBstType(bstType.getType());
            data.setBstId(bill.getBillId());
            data.setAmount(bill.getAmount());
            data.setChannelId(channelId);
            data.setStatus(PayBillStatus.WAIT_PAY.getStatus());
            data.setDescription("计时订单" + bill.getBillNo());


            // TODO 渠道成本及服务费

            // 支付人
            if (TransactionBillPayType.WECHAT.getType().equals(data.getChannelId())) {
                data.setAccount(user.getWxOpenId());
            }

            int insert = this.insertPayBill(data);
            if (insert != 1) {
                return null;
            }

            return this.selectByPayNo(data.getPayNo());
        } finally {
            redisLock.unlock(RedisLockKey.PAY_BILL, lockKey);
        }

    }

    @Override
    public boolean closeByBstId(Long bstId, PayBillBstType bstType) {
        if (bstId == null || bstType == null ) {
            return false;
        }
        PayBillQuery query = new PayBillQuery();
        query.setBstId(bstId);
        query.setBstType(bstType.getType());
        query.setStatusList(PayBillStatus.getCanCancelList());
        List<PayBillVO> payingList = this.selectPayBillList(query);
        if (CollectionUtils.isEmpty(payingList)) {
            return true;
        }

        // 关闭正在支付中的订单
        transactionTemplate.execute(status -> {
            // 修改状态
            PayBill closeData = new PayBillVO();
            closeData.setStatus(PayBillStatus.CANCEL.getStatus());
            int update = this.updateByQuery(closeData, query);
            ServiceUtil.assertion(update != payingList.size(), "修改订单状态失败");

            for (PayBillVO bill : payingList) {
                // 关闭支付中的订单
                if (PayBillStatus.PAYING.getStatus().equals(bill.getStatus())
                        && TransactionBillPayType.WECHAT.getType().equals(bill.getChannelId())) {
                    wxPayService.closeOrder(bill.getPayNo());
                }
            }

            return update;
        });


        return true;
    }

    @Override
    public List<PayBillVO> selectByBstId(Long bstId, String bstType) {
        return payBillMapper.selectByBstId(bstId, bstType);
    }

    @Override
    public PayBillVO selectByPayNo(String payNo) {
        return payBillMapper.selectByPayNo(payNo);
    }

    @Override
    public int updateByQuery(PayBill data, PayBillQuery query) {
        if (data == null || query == null) {
            return 0;
        }
        return payBillMapper.updatePayQuery(data, query);
    }

    @Override
    public void handleSuccess(String payNo, LocalDateTime payTime) {
        if (payNo == null) {
            return;
        }
        PayBillVO payBill = this.selectByPayNo(payNo);
        log.info("获取支付订单:{}", payBill);
        ServiceUtil.assertion(payBill == null, "支付订单不存在");
        ServiceUtil.assertion(!PayBillStatus.PAYING.getStatus().equals(payBill.getStatus()), "该支付订单不是正在支付的支付订单");

        transactionTemplate.execute(status -> {
            // 修改支付订单状态
            PayBill data = new PayBill();
            data.setStatus(PayBillStatus.PAY_SUCCESS.getStatus());
            PayBillQuery query = new PayBillQuery();
            query.setStatus(PayBillStatus.PAYING.getStatus());
            query.setPayId(payBill.getPayId());
            int update = this.updateByQuery(data, query);
            ServiceUtil.assertion(update != 1, "支付订单状态已改变，请稍后再试");

            if (PayBillBstType.TIME_BILL.getType().equals(payBill.getBstType())) {
                // 时长订单
                timeBillService.handleSuccess(payBill);
            }

            return update;
        });
    }
}
