package com.ruoyi.ss.receiveBill.service.impl;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.receiveBill.domain.enums.ReceiveBillStatus;
import com.ruoyi.ss.receiveBill.domain.enums.ReceiveBillType;
import com.ruoyi.ss.transactionBill.service.impl.TransactionBillServiceImpl;
import com.ruoyi.ss.user.domain.SmUserVo;
import com.ruoyi.ss.user.service.ISmUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.ss.receiveBill.mapper.ReceiveBillMapper;
import com.ruoyi.ss.receiveBill.domain.ReceiveBill;
import com.ruoyi.ss.receiveBill.domain.ReceiveBillVO;
import com.ruoyi.ss.receiveBill.domain.ReceiveBillQuery;
import com.ruoyi.ss.receiveBill.service.ReceiveBillService;
import org.springframework.transaction.support.TransactionTemplate;

/**
 * 应收账单Service业务层处理
 *
 * @author ruoyi
 * @date 2024-07-22
 */
@Service
public class ReceiveBillServiceImpl implements ReceiveBillService
{
    @Autowired
    private ReceiveBillMapper receiveBillMapper;

    @Autowired
    private ISmUserService userService;
    @Autowired
    private TransactionBillServiceImpl transactionBillService;
    @Autowired
    private TransactionTemplate transactionTemplate;

    /**
     * 查询应收账单
     *
     * @param billId 应收账单主键
     * @return 应收账单
     */
    @Override
    public ReceiveBillVO selectReceiveBillByBillId(Long billId)
    {
        return receiveBillMapper.selectReceiveBillByBillId(billId);
    }

    /**
     * 查询应收账单列表
     *
     * @param receiveBill 应收账单
     * @return 应收账单
     */
    @Override
    public List<ReceiveBillVO> selectReceiveBillList(ReceiveBillQuery receiveBill)
    {
        return receiveBillMapper.selectReceiveBillList(receiveBill);
    }

    /**
     * 新增应收账单
     *
     * @param receiveBill 应收账单
     * @return 结果
     */
    @Override
    public int insertReceiveBill(ReceiveBill receiveBill)
    {
        receiveBill.setCreateTime(DateUtils.getNowDate());
        return receiveBillMapper.insertReceiveBill(receiveBill);
    }

    /**
     * 修改应收账单
     *
     * @param receiveBill 应收账单
     * @return 结果
     */
    @Override
    public int updateReceiveBill(ReceiveBill receiveBill)
    {
        return receiveBillMapper.updateReceiveBill(receiveBill);
    }

    /**
     * 批量删除应收账单
     *
     * @param billIds 需要删除的应收账单主键
     * @return 结果
     */
    @Override
    public int deleteReceiveBillByBillIds(Long[] billIds)
    {
        return receiveBillMapper.deleteReceiveBillByBillIds(billIds);
    }

    /**
     * 删除应收账单信息
     *
     * @param billId 应收账单主键
     * @return 结果
     */
    @Override
    public int deleteReceiveBillByBillId(Long billId)
    {
        return receiveBillMapper.deleteReceiveBillByBillId(billId);
    }

    @Override
    public int genBillByMonthAndPay(SmUserVo user, DeviceVO device, LocalDateTime billTime, BigDecimal amount) {
        ServiceUtil.assertion(user == null || user.getUserId() == null, "用户不存在");
        ServiceUtil.assertion(device == null || device.getDeviceId() == null, "设备不存在");
        ServiceUtil.assertion(billTime == null, "请指定一个月份");

        // 查询指定月份是否已出账
        ReceiveBillQuery query = new ReceiveBillQuery();
        query.setBillYear(billTime.getYear());
        query.setBillMonth(billTime.getMonthValue());
        query.setType(ReceiveBillType.MONTH.getType());
        query.setUserId(user.getUserId());
        query.setDeviceId(device.getDeviceId());
        int count = this.selectCount(query);
        if ( count > 0) {
            return count;
        }

        // 若未出账则生成账单
        ReceiveBill bill = new ReceiveBill();
        bill.setUserId(user.getUserId());
        bill.setDeviceId(device.getDeviceId());
        bill.setType(ReceiveBillType.MONTH.getType());
        bill.setStatus(ReceiveBillStatus.PAID.getStatus());
        bill.setBillTime(billTime);
        bill.setAmount(amount);
        bill.setDescription(String.format("%s年%s月-设备%s月费", billTime.getYear(), billTime.getMonthValue(), device.getDeviceNo()));
        bill.setReceivedAmount(amount);
        Integer result = transactionTemplate.execute(status -> {
            // 用户余额扣减
            int subtract = userService.subtractBalance(user.getUserId(), amount, false, bill.getDescription());
            ServiceUtil.assertion(subtract != 1, "扣减商户余额失败");

            // 插入账单
            return this.insertReceiveBill(bill);
        });

        return result == null ? 0 : result;
    }

    @Override
    public int selectCount(ReceiveBillQuery query) {
        return receiveBillMapper.selectCount(query);
    }

}
